//+------------------------------------------------------------------+
//|                                                      Leading.mq4 |
//|                                                                  |
//| Leading                                                          |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//+------------------------------------------------------------------+
#property copyright "Coded by Witold Wozniak"
#property link      "www.mqlsoft.com"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Blue

extern double Alpha1 = 0.25;
extern double Alpha2 = 0.33;

double NetLead[];
double Lead[];
double EMA[];

int buffers = 0;
int drawBegin = 0;

int init() {
    drawBegin = 16;
    initBuffer(NetLead, "Lead", DRAW_LINE);
    initBuffer(EMA, "EMA", DRAW_LINE);
    initBuffer(Lead);
    IndicatorBuffers(buffers);
    IndicatorShortName("Leading [" + DoubleToStr(Alpha1, 2) + ", " + DoubleToStr(Alpha2, 2) + "]");  
    return (0);
}
  

int start() {
    if (Bars <= drawBegin) return (0);
    int countedBars = IndicatorCounted();
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, limit = Bars - countedBars - 1;
    for (s = limit; s >= 0; s--) {
        Lead[s] = 2.0 * P(s) + (Alpha1 - 2.0) * P(s + 1) + (1.0 - Alpha1) * Lead[s + 1];
        NetLead[s] = Alpha2 * Lead[s] + (1 - Alpha2) * NetLead[s + 1];
        EMA[s] = 0.5 * P(s) + 0.5 * EMA[s + 1];
    }   
    return (0);   
}

double P(int index) {
    return ((High[index] + Low[index]) / 2.0);
}

void initBuffer(double array[], string label = "", int type = DRAW_NONE, int arrow = 0, int style = EMPTY, int width = EMPTY, color clr = CLR_NONE) {
    SetIndexBuffer(buffers, array);
    SetIndexLabel(buffers, label);
    SetIndexEmptyValue(buffers, EMPTY_VALUE);
    SetIndexDrawBegin(buffers, drawBegin);
    SetIndexShift(buffers, 0);
    SetIndexStyle(buffers, type, style, width);
    SetIndexArrow(buffers, arrow);
    buffers++;
}